package com.cht.tas.callout;

import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PreDestroy;
import java.nio.charset.StandardCharsets;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Component
public class IVRMqtt {
    private final Logger Log = LoggerFactory.getLogger(IVRMqtt.class);

    @Value("${mqtt.phone-conn.calloutResultTopic}")
    private String calloutResultTopic = "";

    @Value("${mqtt.phone-conn.eventTopic}")
    private String eventTopic = "";

    @Value("${mqtt.phone-conn.actionDebugTopic}")
    private String actionDebugTopic = "";

    @Autowired
    private IoTTas ioTTas;

    @Autowired
    private MqttTas mqttTas;

    @Autowired
    private CalloutEvent calloutEvent;

    private ExecutorService publishService = Executors.newSingleThreadExecutor();

    public Map userMap;

    public static int serialNumber = 0;


    public void setSubscribeAllTopic() {
        setSubscribeActionDebugTopic();
        setSubscribeEventTopic();
        setSubscribeCalloutResultTopic();
    }

    //Callout 結果
    public void setSubscribeCalloutResultTopic(){
        mqttTas.setSubscribeTopicResp(calloutResultTopic, (topic, message) -> {
            String msg = new String(message.getPayload(), StandardCharsets.UTF_8);
            JSONObject reqJObjs = new JSONObject(msg);
            Log.debug("MQTT RECV:" + reqJObjs.toString());
        });

    }

    //若 Callout IVR 系統發生錯誤，則印出除錯訊息
    public void setSubscribeActionDebugTopic(){
        mqttTas.setSubscribeTopicResp(actionDebugTopic,((topic, message) -> {
            String msg = new String(message.getPayload(), StandardCharsets.UTF_8);
            Log.debug("DEBUG RECV:" + msg);
        }));
    }

    //設定互動式選單的內容
    public void setSubscribeEventTopic(){
        mqttTas.setSubscribeTopicResp(eventTopic, (topic, message) -> {
            String msg = new String(message.getPayload(), StandardCharsets.UTF_8);
            JSONObject reqJObjs = new JSONObject(msg);
            String type = reqJObjs.getString("type");


            if ("calloutEvent".equals(type)) {
                String event = reqJObjs.getString("event");
                if (event.equals("CallEstablished")) {
                    //呼叫建立
                    Log.debug("CallEstablished");
                } else if (event.equals("CallHangUp")) {
                    //呼叫結束
                    Log.debug("CallHangUp");
                }
            }

            //收到 MQTT Callout 互動選單的 request
            if ( "calloutRequest".equals(type)) {

                //拆解 node 節點的內容及參數
                String[] mArgs = reqJObjs.getString("node").split(",");

                //節點內容
                String node = mArgs[0];

                //節點參數
                String param = "";
                if (mArgs.length > 1) {
                    param = mArgs[1];
                }

                String finalParam = param;
                publishService.submit(()-> {
                    switch (node) {
                        //接通後，進入 MAIN 節點
                        case "MAIN":{
                            calloutEvent.menu(reqJObjs);
                            break;
                        }
                        //節點內容
                        case "MyAskNode":{
                            calloutEvent.myAskNode(reqJObjs, finalParam);
                            break;
                        }
                        default:{
                            Log.debug("default:{}", node);
                        }

                    }
                });
            }
        });
    }

    @PreDestroy
    private void destory(){
        publishService.shutdown();
    }
}
