package com.cht.tas.callout;

import okhttp3.*;
import org.json.JSONArray;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.io.IOException;

@Component
public class IoTTas {
    private static OkHttpClient iotClient;
    private final Logger Log = LoggerFactory.getLogger(IoTTas.class);
    @Value("${mqtt.phone-conn.serviceNumber}")
    private String serviceNumber = "";
    @Value("${mqtt.phone-conn.apiKey}")
    private String apiKey;
    @Value("${phone-conn.callout.url}")
    private String calloutURL = "";

    @PostConstruct
    public void constructor() {

        Log.debug("URL:{}", calloutURL);
        Log.debug("serviceNumber:{}", serviceNumber);
        Log.debug("AuthKey:{}", apiKey);

        iotClient = new OkHttpClient.Builder()
//                .connectTimeout(15, TimeUnit.SECONDS)
//                .readTimeout(15, TimeUnit.SECONDS)
//                .retryOnConnectionFailure(true)
                .build();

        Log.debug("timeout:{}", iotClient.connectTimeoutMillis());
    }


    public String calloutIVR(JSONArray phones) {
        return "{\n" +
                "   \"serviceNumber\":\"0281920408\",\n" +
                "   \"phones\":" + phones.toString() + ",\n" +
                "   \"ivrData\":{\n" +
                "      \"node\":\"MAIN\",\n" +
                "      \"nextNode\":\"CUSTOM\"\n" +
                "   }\n" +
                "}";
    }

    public void callout(String content) {
        MediaType type = MediaType.parse("application/json; charset=utf-8");
        RequestBody reqBody = RequestBody.create(type, content);

        Log.debug("URL:{}", calloutURL);

        Request request = new Request.Builder()
                .url(calloutURL)
                .addHeader("X-API-Key", apiKey)
                .post(reqBody)
                .build();

        Log.debug("do callout request");
        iotClient.newCall(request).enqueue(new Callback() {

            @Override
            public void onFailure(Call call, IOException e) {
                Log.debug("error:" + e.getLocalizedMessage());
            }

            @Override
            public void onResponse(Call call, Response response) {
                Log.debug("code:" + response.code());
            }
        });
    }

}
